import os.path

import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from numpy import count_nonzero


class Interviews:
    """
    This class shows data points of every saved interview.
    In this data frame is also saved how much time was taken for the questionnaire and some additional variables as you
    can see here: https://www.soscisurvey.de/help/doku.php/en:results:variables
    """

    def __init__(self) -> None:
        super().__init__()
        self.df = pd.read_csv('../data/data_ueq.csv', delimiter=',', quotechar='"', encoding='utf-16')

    def __str__(self) -> str:
        return f'{self.df.shape[0]} interviews'

    def get_items_per_product_category(self, product_category_items):
        return self.df[product_category_items]

    def get_filtered(self, all_categories):
        if isinstance(all_categories, ProductCategories):
            cat_games = all_categories.get_items('games')
            cat_lms = all_categories.get_items('lms')
            # remove interviews which are not complete
            print(cat_games)
            print(self.df)
            unwanted_interviews = self.df[self.df[cat_games[0]].isnull() | self.df[cat_lms[0]].isnull()]
            print(f'{unwanted_interviews.shape[0]} interviews have not completed the questionnaire.')
            df_nonnull_interviews = self.df.drop(unwanted_interviews.index)
            # remove interviews which are, based on indicators, not well answered
            df_too_fast_interviews = df_nonnull_interviews[
                (df_nonnull_interviews['TIME_RSI'] > 2.0) | (df_nonnull_interviews['DEG_TIME'] > 150)]
            print(f'{df_too_fast_interviews.shape[0]} interviews have completed too fast or too weird.')
            df_filtered = df_nonnull_interviews.drop(df_too_fast_interviews.index)
            print(f'{df_filtered.shape[0]} filtered interviews.')
            return df_filtered


class Variables:
    """
    Gives an explanation of every variable used in the questionnaire.
    """

    def __init__(self) -> None:
        super().__init__()
        self.df = pd.read_csv('../data/variables_ueq.csv', delimiter=';', quotechar='"', encoding='utf-16')

    def get_groups(self):
        return self.df[['VAR', 'LABEL']][7:12]


class Values:
    """
    Gives an explanation of every given value in the interviews
    """

    def __init__(self) -> None:
        super().__init__()
        self.df = pd.read_csv('../data/values_ueq.csv', delimiter=';', quotechar='"', encoding='utf-16')


class ProductCategories:
    """
    Product categories are the core of this evaluation.
    The goal is to evaluate which items are important for the product categories.
    In the created dict in items every product category is connected with the used code and names of their items.
    """

    def __init__(self) -> None:
        super().__init__()
        self.items = {
            'dashboard': {
                'code': 'EX01'
            },
            'games': {
                'code': 'EX05'
            },
            'lms': {
                'code': 'EX07'
            },
            'vr': {
                'code': 'EX08'
            },
        }

        # create the tokens to access every item in a product category
        for key in self.items.keys():
            # something went wrong in the creation of the catalogue so there are different numbers for the VR items
            if key == 'vr':
                self.items[key]['items'] = [self.items[key]['code'] + "_" + "{:02d}".format(i) for i in range(21, 41)]
            else:
                self.items[key]['items'] = [self.items[key]['code'] + "_" + "{:02d}".format(i) for i in range(1, 21)]

    def get_items(self, key):
        return self.items[key]['items']


def save_plot(plot, path):
    dir_path = os.path.abspath('../results/')
    if not os.path.exists(dir_path):
        os.mkdir(dir_path)
    plot.savefig(fname=os.path.join(dir_path, path + '.png'), dpi='figure', format='png')


def replace_cryptic_names(columns):
    new_columns = list(columns)
    for i in range(len(columns)):
        if "_01" in new_columns[i]:
            new_columns[i] = 'Inhaltsqualität'
        elif "_02" in new_columns[i]:
            new_columns[i] = 'Anpassbarkeit'
        elif "_03" in new_columns[i]:
            new_columns[i] = 'Durchschaubarkeit'
        elif "_04" in new_columns[i]:
            new_columns[i] = 'Effizienz'
        elif "_05" in new_columns[i]:
            new_columns[i] = 'Immersion'
        elif "_06" in new_columns[i]:
            new_columns[i] = 'Intuitive Bedienung'
        elif "_07" in new_columns[i]:
            new_columns[i] = 'Nützlichkeit'
        elif "_08" in new_columns[i]:
            new_columns[i] = 'Originalität'
        elif "_09" in new_columns[i]:
            new_columns[i] = 'Schönheit'
        elif "_10" in new_columns[i]:
            new_columns[i] = 'Identität'
        elif "_11" in new_columns[i]:
            new_columns[i] = 'Steuerbarkeit'
        elif "_12" in new_columns[i]:
            new_columns[i] = 'Stimulation'
        elif "_13" in new_columns[i]:
            new_columns[i] = 'Übersichtlichkeit'
        elif "_14" in new_columns[i]:
            new_columns[i] = 'Verbundenheit'
        elif "_15" in new_columns[i]:
            new_columns[i] = 'Vertrauen'
        elif "_16" in new_columns[i]:
            new_columns[i] = 'Wertigkeit'
        elif "_17" in new_columns[i]:
            new_columns[i] = 'Attraktivität'
        elif "_18" in new_columns[i]:
            new_columns[i] = 'Haptik'
        elif "_19" in new_columns[i]:
            new_columns[i] = 'Akustik'
        elif "_20" in new_columns[i]:
            new_columns[i] = 'Vertrauenswürdigkeit'

    return new_columns


def filter_columns_by_value(df_dashboard, value):
    filtered_columns = df_dashboard.columns[value]
    df_dashboard_area = df_dashboard[filtered_columns]
    return df_dashboard_area


class Plotter:

    def __init__(self, df) -> None:
        super().__init__()
        self.df = df

    def plot_groups(self, variable_labels):
        df_groups = self.df[['DE01_01', 'DE01_02', 'DE01_03', 'DE01_04', 'DE01_05']] - 1
        print(variable_labels)

        df_groups.columns = ['Stud', 'Doz', 'WiMi', 'TeVe', 'NU']
        sns.set_theme(style="whitegrid")
        ax = sns.barplot(data=df_groups, palette="Blues_d", estimator=count_nonzero, ci=None)
        ax.set(title='Statusgruppen')
        sns.despine(left=True)
        save_plot(plt, 'distributions_by_groups')
        plt.show()

    def plot_sex(self):
        df_sex = self.df['DE02'] \
            .replace(1, 'male') \
            .replace(2, 'female') \
            .replace(3, 'diverse') \
            .replace(-1, 'not mentioned')
        ax = sns.countplot(x=df_sex, palette="Blues_d")
        ax.set(title='Geschlecht')
        print(df_sex.value_counts(normalize=True))
        save_plot(plt, 'distribution_by_gender')
        plt.show()

    def plot_faculty(self):
        df_faculty = self.df['DE03'] \
            .replace(1, 'Jur') \
            .replace(2, 'Phil') \
            .replace(3, 'HuWi') \
            .replace(4, 'WiSo') \
            .replace(5, 'MathNat') \
            .replace(6, 'Ges') \
            .replace(7, 'HPI') \
            .replace(8, 'ZE') \
            .replace(9, 'Other') \
            .replace(-1, 'kA') \
            .replace(-9, 'nB')
        ax = sns.countplot(x=df_faculty, palette="Blues_d")
        ax.set(title='Fakultät')
        print(df_faculty.value_counts(normalize=True))
        save_plot(plt, 'distribution_by_faculty')
        plt.show()

    def plot_item(self, category_items, category_title, area):
        df_dashboard = self.df[category_items]
        if area == 'top':
            df_dashboard_area = filter_columns_by_value(df_dashboard, df_dashboard.mean() >= 5)
        elif area == 'mid':
            df_flc = filter_columns_by_value(df_dashboard, df_dashboard.mean() < 5)
            df_dashboard_area = filter_columns_by_value(df_flc, df_flc.mean() >= 3)
        elif area == 'low':
            df_dashboard_area = filter_columns_by_value(df_dashboard, df_dashboard.mean() < 3)
        else:
            df_dashboard_area = df_dashboard
        print('------------- ' + area)
        df_dashboard_area.columns = replace_cryptic_names(df_dashboard_area.columns)
        df_dashboard_area = df_dashboard_area.reindex(df_dashboard_area.mean().sort_values(ascending=False).index,
                                                      axis=1)
        print(df_dashboard_area.columns)
        ax = sns.barplot(data=df_dashboard_area, palette="Blues_d", orient='h', ci="sd")
        ax.set(title=category_title + ' - ' + area)
        sns.despine(left=True)
        save_plot(plt, 'items_dashboard_' + area)
        plt.show()


def setup_dataframes():
    return Interviews(), Variables(), Values(), ProductCategories()


if __name__ == '__main__':
    interview, variables, values, product_categories = setup_dataframes()
    print(interview)
    df_filtered = interview.get_filtered(product_categories)
    plotter = Plotter(df_filtered)
    plotter.plot_groups(variables.get_groups())
    plotter.plot_sex()
    plotter.plot_faculty()
    plotter.plot_item(product_categories.get_items('dashboard'), 'dashboard', 'top')
    plotter.plot_item(product_categories.get_items('dashboard'), 'dashboard', 'mid')
    plotter.plot_item(product_categories.get_items('dashboard'), 'dashboard', 'low')
